%% Estimador de sistema SISO no observable 
echo off
clear
close all
warning off
%% Modelo de estado
A = [1 1 0;0 -0.5 2.9;0 -1 0.9];
B = [1 1 0]';
C = [0 0 1];
D = 0;
siso_mdl = ss(A,B,C,D)
if (1-isstable(siso_mdl))
    fprintf('El modelo es inestable\n')
end
%% Observabilidad
V=obsv(siso_mdl)
if (rank(V) == length(A))
    fprintf('El modelo es observable\n')
else
    fprintf('El modelo es no observable\n')
end
%% Matriz transformacin 
Q = [V(1:2,:); 1 0 0];
At = Q*A/Q;
Bt = Q*B;
Ct = C/Q;
siso_mdl_t = ss(At,Bt,Ct,D)
% Como A_22 es 1, su autovalor es 1 y por lo tanto no es detectable.
%% Estimador utilizando A_11
Ato = At(1:2,1:2);
c1 = Ct(1:2);
b1 = Bt(1:2);
d1 = 0;
red_mdl = ss(Ato,b1,c1,d1)
%% MCO
[num,den]=tfdata(red_mdl,'v');
Ac=[0,1;-den(3),-den(2)];
Bc=[0;1];
Cc=[num(3),num(2)];
Dc=0;
clc;
fprintf('\n Modelo Cannico Observable \n');
mco_red_mdl=transpose(ss(Ac,Bc,Cc,Dc))


%% Matriz de transformacin de modelos
clc;
fprintf('\n Matriz Observabilidad de la Planta Original \n');
V1=obsv(red_mdl)
fprintf('\n Matriz Observabilidad del Modelo Canonico \n');
V2=obsv(mco_red_mdl)
fprintf('\n Matriz Transformacion al Modelo Canonico \n');
R=V1\V2
P5=ss2ss(red_mdl,R\eye(length(R)))
%% Autovalores del estimador
z = [0 0]';
%%
H1=acker(mco_red_mdl.a',mco_red_mdl.c',z)'
fprintf('\n Vector del Estimador transformado desde MCO \n');
H2=R*H1
H = [H2; 0];
Ho = Q\H;
Ae = A-Ho*C;
Be = [B Ho];
Ce = eye(3);
De = zeros(3,2);
siso_est_mdl = ss(Ae,Be,Ce,De)
%% Estimador MIMO
clear
%% Modelo de estado
A = [1 1 0;0 -0.5 2.9;0 -1 0.9];
B = [1 1 0]';
C = [1 0 0;0 0 1];
D = 0;
mimo_mdl = ss(A,B,C,D)
if (1-isstable(mimo_mdl))
    fprintf('El modelo es inestable\n')
end
%% Observabilidad
V=obsv(mimo_mdl)
if (rank(V) == length(A))
    fprintf('El modelo es observable\n')
else
    fprintf('El modelo es no observable\n')
end
%% Transformacin a SISO equivalente
F = [1 1];
Cm = F*C;
siso_eq_mdl = ss(A,B,Cm,D)
%% Observabilidad SISO equivalente
V=obsv(siso_eq_mdl)
if (rank(V) == length(A))
    fprintf('El modelo es observable\n')
else
    fprintf('El modelo es no observable\n')
end
%% MCO
[num,den]=tfdata(siso_eq_mdl,'v');
Ac=[0,1,0;0 0 1;-den(4), -den(3),-den(2)];
Bc=[0;0;1];
Cc=[num(4),num(3),num(2)];
Dc=0;
clc;
fprintf('\n Modelo Cannico Observable \n');
mco_siso_eq_mdl=transpose(ss(Ac,Bc,Cc,Dc))


%% Matriz de transformacin de modelos
clc;
fprintf('\n Matriz Observabilidad de la Planta Original \n');
V1=obsv(siso_eq_mdl)
fprintf('\n Matriz Observabilidad del Modelo Canonico \n');
V2=obsv(mco_siso_eq_mdl)
fprintf('\n Matriz Transformacion al Modelo Canonico \n');
R=V1\V2
P5=ss2ss(siso_eq_mdl,R\eye(length(R)))

%% Autovalores del estimador
z = [0 0 0]';
%%
H1=acker(mco_siso_eq_mdl.a',mco_siso_eq_mdl.c',z)'
H2=R*H1
Hm = H2*F
Ae = A-Hm*C;
Be = [B Hm];
Ce = eye(3);
De = zeros(3,3);
mimo_est_mdl = ss(Ae,Be,Ce,De)
